"use strict";
require("reflect-metadata");
var Registry_1 = require("./Registry");
Registry_1.Registry.setInjectorFactory(function () { return new Injector(); });
var Injector = (function () {
    function Injector() {
        this.cached = {};
    }
    Injector.prototype.get = function (name) {
        return this._get(name, new Set());
    };
    Injector.prototype.has = function (name) {
        return !!this.cached[name] || !!Registry_1.Registry.getMetadata(name);
    };
    Injector.prototype.invoke = function (factory, self, locals) {
        var previous = new Set();
        return this._invoke(factory, previous, self, locals);
    };
    Injector.prototype._invoke = function (factory, previous, self, locals) {
        var _this = this;
        var fn = this._getInjectableFunction(factory);
        var $inject = fn.$inject;
        var args = $inject ? $inject.map(function (k) { return _this._get(k, previous, locals); }) : [];
        return fn.apply(self, args);
    };
    Injector.prototype.instantiate = function (type, locals) {
        var previous = new Set();
        var result = this._instantiate(type, previous, locals);
        this._injectProperties(result, type, previous, locals);
        return result;
    };
    Injector.prototype._instantiate = function (type, previous, locals) {
        var _this = this;
        var fn = this._getInjectableFunction(type);
        var $inject = fn.$inject;
        var args = $inject ? $inject.map(function (k) { return _this._get(k, previous, locals); }) : [];
        return new (type.bind.apply(type, [void 0].concat(args)))();
    };
    Injector.prototype._injectProperties = function (result, type, previous, locals) {
        var props = Registry_1.Registry.getPropertyMetaData(type) || {};
        var keys = Object.keys(props);
        for (var _i = 0, keys_1 = keys; _i < keys_1.length; _i++) {
            var prop = keys_1[_i];
            result[prop] = this._get(props[prop].name, previous, locals);
        }
        var postConstruct = Registry_1.Registry.getPostConstructMetaData(type);
        if (postConstruct) {
            result[postConstruct.propertyKey]();
        }
    };
    Injector.prototype._get = function (name, previous, locals) {
        var result = this.cached[name];
        if (result) {
            return result;
        }
        if (previous.has(name)) {
            throw "inject: '" + name + "' has circular reference";
        }
        previous.add(name);
        var data = Registry_1.Registry.getMetadata(name);
        if (!data) {
            if (locals) {
                result = locals[name];
                if (result) {
                    return result;
                }
            }
            throw "inject: '" + name + "' is not registered";
        }
        if (data.type) {
            result = this._instantiate(data.type, previous, locals);
        }
        else if (data.factory) {
            result = this._invoke(data.factory, previous, null, locals);
        }
        else {
            result = data.value;
        }
        this.cached[name] = result;
        if (data.type) {
            this._injectProperties(result, data.type, previous, locals);
        }
        return result;
    };
    Injector.prototype._getInjectableFunction = function (fn) {
        if (fn instanceof Array) {
            var last = fn.pop();
            last.$inject = fn;
            return last;
        }
        else if (typeof fn === 'function') {
            var $inject_1;
            if (!($inject_1 = fn.$inject)) {
                $inject_1 = [];
                var fnText = fn.toString().replace(Injector.STRIP_COMMENTS, '');
                var argDecl = fnText.match(Injector.FN_ARGS);
                for (var _i = 0, _a = argDecl[1].split(Injector.FN_ARG_SPLIT); _i < _a.length; _i++) {
                    var arg = _a[_i];
                    arg.replace(Injector.FN_ARG, function (str, _, name) {
                        $inject_1.push(name);
                        return str;
                    });
                }
                fn.$inject = $inject_1;
                return fn;
            }
        }
        return fn;
    };
    return Injector;
}());
Injector.FN_ARGS = /^function\s*[^\(]*\(\s*([^\)]*)\)/m;
Injector.FN_ARG_SPLIT = /,/;
Injector.FN_ARG = /^\s*(_?)(.+?)\1\s*$/;
Injector.STRIP_COMMENTS = /((\/\/.*$)|(\/\*[\s\S]*?\*\/))/mg;
exports.Injector = Injector;
